
#include <itersolver.h>
#include <fopmerge.h>

IterSolver::~IterSolver(){
}

// solve function uses zero initial guess and calls derived class solver
floatspacearray IterSolver::Solve( const floatspacearray & rhs ) {
   floatspacearray init = op->Adjoint( rhs ); init = 0;
   return Solve(  rhs, init );
}


// report on progress
void IterSolver::report(){
    if( repstream != nil ){
          *repstream << "iteration: "<< numiter <<
            "   residual norm: "<< resid_norm <<
             "   norm ratio: "<< resid_norm/rhs_norm <<endl;
     } 
}

// Store one residual in internal array
void IterSolver::save_one_resid(  const floatspacearray & x ){
  if( !save_resid ) return;
	// save every "save_resid'th" iteration 
  if( numiter%save_resid == 0 ){
	saved_resid[numiter/save_resid] = new floatspacearray(x);
  }
}

// Store one interation in internal array
void IterSolver::save_one_iter( const floatspacearray & x ){
  if( !save_x_iter ) return;
	// save every "save_x_iter'th" iteration 
  if( numiter%save_x_iter == 0 ){
	saved_x_iter[numiter/save_x_iter] = new floatspacearray(x);
  }
}

void IterSolver::SetSaveIterStep( int save_step ){
   int num_save;
   save_x_iter = save_step;
   num_save = (maxiter+save_step-1)/save_step;
   saved_x_iter = new floatspacearray*[num_save];
   for( int i=0; i<num_save; saved_x_iter[i] = nil, i++ );
}

void IterSolver::SetSaveResidStep( int save_step ){
   int num_save;
   save_resid = save_step;
   num_save = (maxiter+save_step-1)/save_step;
   saved_resid = new floatspacearray*[num_save];
   for( int i=0; i<num_save; saved_resid[i] = nil, i++ );
}

floatspacearray IterSolver::GetResiduals(){
  // we make a spacearray the same size as one resdual
  // each element of the array is the merge of all the residuals 
  // we saved

  if( !save_resid){
	cerr<<" No resiuals saved, GetResiduals() is invalid \n";
	exit(-1); 
  }

  // figure out how many we saved
  int nsaved = numiter/save_resid;

  // the extra axis we will add to each space
  Axis iterax(0,1,nsaved,"Iteration");
  // Make a merge operator
  fopMerge merger( iterax );

  // the output spacearray
  int nrow = saved_resid[0]->num_rows();
  int ncol = saved_resid[0]->num_cols();
  floatspacearray ret(nrow,ncol);

  for( int i=0; i<ncol; i++ ){
	for( int j=0; j<nrow; j++ ){
	    // make a column array of this element for all saved iters.
	    floatspacearray rescol( nsaved );
	    for( int k=0; k<nsaved; k++ ){
		rescol(k) = (*(saved_resid[k]))(j,i);
	    }
	    // merge them
	    ret(j,i) =  merger.Forward( rescol )(0,0);
	}
  }
  return ret;
}

floatspacearray IterSolver::GetOneResid(int num){

  if( !save_resid){
	cerr<<" No iterations saved, GetOneResid() is invalid \n";
	exit(-1); 
  }

  if( num%save_resid != 0 ){
	cerr<<" Residual number: "<<num<<" was not saved, GetOneResid() \n";
	exit(-1); 
  }

  return *saved_resid[numiter/save_resid];
}

floatspacearray IterSolver::GetOneIter(int num){

  if( !save_x_iter){
	cerr<<" No iterations saved, GetOneIter() is invalid \n";
	exit(-1); 
  }

  if( num%save_x_iter != 0 ){
	cerr<<" Iteration number: "<<num<<" was not saved, GetOneIter() \n";
	exit(-1); 
  }

  return *saved_x_iter[numiter/save_x_iter];
}

floatspacearray IterSolver::GetIterations(){
  // we make a spacearray the same size as one iteration
  // each element of the array is the merge of all the iterations 
  // we saved

  if( !save_x_iter){
	cerr<<" No iterations saved, GetIterations() is invalid \n";
	exit(-1); 
  }

  // figure out how many we saved
  int nsaved = numiter/save_x_iter;

  // the extra axis we will add to each space
  Axis iterax(0,1,nsaved,"Iteration");
  // Make a merge operator
  fopMerge merger( iterax );

  // the output spacearray
  int nrow = saved_x_iter[0]->num_rows();
  int ncol = saved_x_iter[0]->num_cols();
  floatspacearray ret(nrow,ncol);

  for( int i=0; i<ncol; i++ ){
	for( int j=0; j<nrow; j++ ){
	    // make a column array of this element for all saved iters.
	    floatspacearray rescol( nsaved );
	    for( int k=0; k<nsaved; k++ ){
		rescol(k) = (*(saved_x_iter[k]))(j,i);
	    }
	    // merge them
	    ret(j,i) =  merger.Forward( rescol )(0,0);
	}
  }
  return ret;
}
